%
% Princeton University, COS 429, Fall 2016
%
% tinynet_sgd.m
%   Trains a tiny (2 hidden-node + 1 output) network with SGD
%
% Inputs:
%   X: datapoints (one per row, should include a column of ones
%                  if the model is to have a constant)
%   z: ground-truth labels (0/1)
%   num_epochs: number of epochs to train over
% Output:
%   params: vector of parameters 
%

function params = tinynet_sgd(X, z, num_epochs)

    [num_pts, num_vars] = size(X);

    % Indices into params of weights for hidden nodes u and v, as well as output
    ubegin = 1;
    uend = num_vars;
    vbegin = uend + 1;
    vend = uend + num_vars;
    obegin = vend + 1;
    oend = vend + 3;

    % Initial (random) estimate of params.
    mean = 0;
    usigma = 1 / sqrt(num_vars / 2);
    vsigma = 1 / sqrt(num_vars / 2);
    osigma = 1 / sqrt(3 / 2);
    params = zeros(oend, 1);
    params(ubegin:uend) = normrnd(mean, usigma, num_vars, 1);
    params(vbegin:vend) = normrnd(mean, vsigma, num_vars, 1);
    params(obegin:oend) = normrnd(mean, osigma, num_vars, 1);

    % Loop over epochs
    for ep = 1:num_epochs

        % Permute the data rows
        permutation = randperm(num_pts);
        X = X(permutation, :);
        z = z(permutation);

        % Iterate over the points
        for i = 1:num_pts

            % Forward propagation pass to evaluate the network
            u = relu(X(i,:) * params(ubegin:uend));
            v = relu(X(i,:) * params(vbegin:vend));
            z_hat = logistic([1 u v] * params(obegin:oend));

            % Backward propagation pass to evaluate gradient
            gradient = zeros(oend, 1);

            gradient(...) = ...

            % Gradient descent step
            params = ...

        end

    end

end

%
% The logistic "sigmoid" function
%

function val = logistic(x)
    val = 1 ./ (1 + exp(-x));
end

%
% The "RELU" function
%

function val = relu(x)
    val = max(x, 0);
end

